searchNodes=[{"doc":"ExRunnerElixir library that provides a macro which converts the modules into operations for encapsulating business logics. It uses Ecto Schema (embedded_schema) for defining input / output and Ecto Changeset for the validations. :)","ref":"readme.html","title":"ExRunner","type":"extras"},{"doc":"It can be installed by adding ex_runner to your list of dependencies in mix.exs:def deps do [ {:ex_runner, &quot;~&gt; 0.1.0&quot;} ] end","ref":"readme.html#installation","title":"ExRunner - Installation","type":"extras"},{"doc":"Let's take an example to understand how this library works. Example - Write an operation which creates the user session by taking email and password.Step 1 - Define a module and add use ExRunner.defmodule CreateSession do use ExRunner endStep 2 - Define input and output for the operation. Input is what will be given as parameters to this operation and Output is what will get returned from the operation as a result.defmodule CreateSession do use ExRunner input do field :email, :string field :password, :string end output do field :session_id, Ecto.UUID end endThis is just Ecto Schema For complex schema definition, embeds_one and embeds_many can be used.Step 3 - Define validate function which adds the required validations on the inputs passed to the operation.defmodule CreateSession do use ExRunner input do field :email, :string field :password, :string end output do field :session_id, Ecto.UUID end defp validate(changeset) do changeset |&gt; validate_required([:email, :password]) |&gt; validate_format(:email, ~r/^[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,4}$/) |&gt; validate_length(:password, min: 4) end endValidate function takes changeset as argument, changes of which contains the input passed and schema is of type input defined in step 2.For complex validations, read Ecto ChangesetStep 4- Define execute function which performs business logic with the inputs provided. defmodule CreateSession do use ExRunner input do field :email, :string field :password, :string end output do field :session_id, Ecto.UUID end defp validate(changeset) do changeset |&gt; validate_required([:email, :password]) |&gt; validate_format(:email, ~r/^[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,4}$/) |&gt; validate_length(:password, min: 4) end defp execute(changeset) do params = changeset.changes case (params.email == &quot;test@test.com&quot; and params.password == &quot;test&quot;) do true -&gt; %{session_id: &quot;870df8e8-3107-4487-8316-81e089b8c2cf&quot;} false -&gt; add_error(changeset, :credentials, &quot;are invalid&quot;) end end endExecute function takes changeset as argument. Inputs / parameters can be found in changeset.changes. To add an error, just call add_error of Ecto.Changeset. In case of :ok, return the needed response of type output defined in step 4. In case of :error, return Ecto.Changeset.Step 5 - Try running the operation.# run with valid credentials &gt; CreateSession.run(email: &quot;test@test.com&quot;, password: &quot;test&quot;) {:ok, %{session_id: &quot;870df8e8-3107-4487-8316-81e089b8c2cf&quot;}} # run! with valid credentials &gt; CreateSession.run!(email: &quot;test@test.com&quot;, password: &quot;test&quot;) %{session_id: &quot;870df8e8-3107-4487-8316-81e089b8c2cf&quot;} # run with invalid email format &gt; CreateSession.run(email: &quot;test&quot;, password: &quot;test&quot;) {:error, #Ecto.Changeset&lt; action: nil, changes: %{email: &quot;test&quot;, password: &quot;testi&quot;}, errors: [email: {&quot;has invalid format&quot;, [validation: :format]}], data: #CreateSession.Input&lt;&gt;, valid?: false &gt;} # run! with invalid email format &gt; CreateSession.run(email: &quot;test&quot;, password: &quot;test&quot;) # raises Ecto.InvalidChangesetError # run with invalid credentials &gt; CreateSession.run(email: &quot;test@test.com&quot;, password: &quot;test1&quot;) {:error, #Ecto.Changeset&lt; action: nil, changes: %{email: &quot;test@test.com&quot;, password: &quot;testi&quot;}, errors: [credentials: {&quot;are invalid&quot;, []}], data: #CreateSession.Input&lt;&gt;, valid?: false &gt;} # run! with invalid credentials &gt; CreateSession.run!(email: &quot;test@test.com&quot;, password: &quot;test1&quot;) # raises Ecto.InvalidChangesetError I recommend reading How does the library work internally ? to understand in detail.","ref":"readme.html#usage","title":"ExRunner - Usage","type":"extras"},{"doc":"run can be called either with a keyword list or a map.1 - It first filters the input and permits only the ones defined in input schema. This is also true for embeds_one and embeds_many.2 - It checks the input against the field types defined in input. If invalid, it returns {:error, changeset}3 - It calls validate which has been defined in the module. If invalid, it returns {:error, changeset}4 - It calls execute which has been defined in the module. If execute returns changeset, it returns {:error, changeset}. If execute returns other than changeset, it stores it as output.5 - It filters the output and permits only the ones defined in output schema. This is also true for embeds_one and embeds_many.6 - It checks the output against the field types defined in output. If invalid, it raises Ecto.InvalidChangesetError.7 - It returns the output as map finally. {:ok, output}In case of run!, if the returned tuple is of {:error}, it raises errors. if the returned tuple is of {:ok}, it returns output.","ref":"readme.html#how-does-the-library-work-internally","title":"ExRunner - How does the library work internally ?","type":"extras"},{"doc":"Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"First Release","ref":"changelog.html#v0-0-1","title":"Changelog - v0.0.1","type":"extras"},{"doc":"MIT License Copyright (c) [2020] [Nishant Dalmia] Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"}]